#!/usr/bin/env python
# coding: utf-8

# # File format example
# 
# 0 : 2022-01-25T11:25:36.000+00:00
# 1 : Coupon,GPC2149_X_001_A_QA
# 2 : Measured by,Ashley,University of Liverpool
# 3 : LabVIEW Version,V3.3,Arduino Version,V3.6b
# 4 : After Reflow, Reflow Cycles,10
# 5 : Measurement Type,Test Coupon
# 6 : Target Tolerance,20%
# 7 : Channel,AVG R [Ohm],s_R [Ohm],R Target [Ohm],Pass/Fail,Min Meas [Ohm],Max Meas. [Ohm],# Low,# In,# Hi,# Readings
# 8 : s0,3.054,0.000,3.028,pass,3.053,3.056,0,100,0,100
# 9 : s1,0.415,0.000,0.411,pass,0.414,0.415,0,100,0,100
# 10 : s2,1.283,0.000,1.272,pass,1.283,1.283,0,100,0,100
# 11 : 
# 12 : Stray Capacitance: 134.47;133.85;84.9
# 13 : Channel,AVG C [pF],s_C [pF],C Target [pF],Pass/Fail,Min Meas [pF],Max Meas. [pF],# Low,# In,# Hi,# Readings
# 14 : c1,65.83,0.00,65.10,pass,65.83,65.83,0,100,0,100
# 15 : c2,152.94,0.09,153.56,pass,152.34,152.95,0,100,0,100
# 16 : c3,82.71,0.00,82.36,pass,82.71,82.71,0,100,0,100

# In[ ]:


import os
import sys
import argparse
import json
from glob import glob

from pathlib import Path
import datetime
import statistics

def analyse (data_file, db_ID, alternative_ID, component_stage, run_number, out_file_name):
    """This function analyses Liverpool pull-test csv files and
    creates a ITk DB json file for the PULL_TEST
    Output file name can be set by calling function"""
    
    print('Start csv file analysis')
    print(f"Data file: {data_file}\n")
    
      
        
    if out_file_name : print(f"Output file name {out_file_name}")
        
    #########Read File########################################
    with open(data_file, 'r') as d_file:
        lines = d_file.readlines()
        
        lines = [line.rstrip().split(',') for line in lines]  #strip newlines and spaces at end of line, split into lists    
        #print(lines)
    
    
    #set from file
    datetimeobject = datetime.datetime.fromisoformat(lines[0][0])
    operator = lines[2][1]
    software = lines[3][0] + ' ' + lines[3][1] + '; ' + lines[3][2] + ' ' + lines[3][3]
    
    
    
    #set from file if not pre-set by user
    if not db_ID :
        db_ID = lines[1][1]   
    if not component_stage :
        if lines[4][0] == "After Reflow" : component_stage = "AFTER_REFLOW"
        if lines[4][0] == "Before Reflow" : component_stage = "FROM_MANUFACTURER"

    #print( "DB ID: ", db_ID, "\tStage: ", component_stage)
    
    resistance_average = [lines[8][1]]
    resistance_average.append(lines[9][1])
    resistance_average.append(lines[10][1])
    
    resistance_stdev = [lines[8][2]]
    resistance_stdev.append(lines[9][2])
    resistance_stdev.append(lines[10][2])
    
    resistance_target = [lines[8][3]]
    resistance_target.append(lines[9][3])
    resistance_target.append(lines[10][3])
    
    resistance_pass = []
    resistance_pass.append(True) if lines[8][4]=="pass" else resistance_pass.append(False)
    resistance_pass.append(True) if lines[9][4]=="pass" else resistance_pass.append(False)
    resistance_pass.append(True) if lines[10][4]=="pass" else resistance_pass.append(False)
    
    resistance_minimum = [lines[8][5]]
    resistance_minimum.append(lines[9][5])
    resistance_minimum.append(lines[10][5])
    
    resistance_maximum = [lines[8][6]]
    resistance_maximum.append(lines[9][6])
    resistance_maximum.append(lines[10][6])
    
    resistance_measurements = [lines[8][10]]
    resistance_measurements.append(lines[9][10])
    resistance_measurements.append(lines[10][10])


    
    
    capacitance_average = [lines[14][1]]
    capacitance_average.append(lines[15][1])
    capacitance_average.append(lines[16][1])
    
    capacitance_stdev = [lines[14][2]]
    capacitance_stdev.append(lines[15][2])
    capacitance_stdev.append(lines[16][2])
    
    capacitance_target = [lines[14][3]]
    capacitance_target.append(lines[15][3])
    capacitance_target.append(lines[16][3])
    
    capacitance_pass = []
    capacitance_pass.append(True) if lines[14][4]=="pass" else capacitance_pass.append(False)
    capacitance_pass.append(True) if lines[15][4]=="pass" else capacitance_pass.append(False)
    capacitance_pass.append(True) if lines[16][4]=="pass" else capacitance_pass.append(False)
    
    capacitance_minimum = [lines[14][5]]
    capacitance_minimum.append(lines[15][5])
    capacitance_minimum.append(lines[16][5])
    
    capacitance_maximum = [lines[14][6]]
    capacitance_maximum.append(lines[15][6])
    capacitance_maximum.append(lines[16][6])
    
    capacitance_measurements = [lines[14][10]]
    capacitance_measurements.append(lines[15][10])
    capacitance_measurements.append(lines[16][10])
    
    #print(capacitance_pass)
    passed = True
    for value in resistance_pass:
        if value == False : passed = False
        else: continue
    for value in capacitance_pass:
        if value == False : passed = False
        else: continue
    
    #print(passed)
    

    
    #########create dictionary for json file######################
    out_dictionary = {}  ##dictionary for output
    
    out_dictionary.update({"componentType": "HYBRID_FLEX_TEST_COUPON"}) 
    out_dictionary.update({"component": db_ID})             
    out_dictionary.update({"alternativeIdentifier": alternative_ID})    
    out_dictionary.update({"currentStage": component_stage})        
    out_dictionary.update({"testType" : "RESISTANCE_CAPACITANCE"})
    out_dictionary.update({"institution" : "LIV"})
    out_dictionary.update({"runNumber" : run_number})                           
    out_dictionary.update({"date" : datetimeobject.astimezone().isoformat(timespec='milliseconds')})      
    out_dictionary.update({"passed" : passed})
    out_dictionary.update({"problems" : False})
    out_dictionary.update({"properties":{
        "TEST_OPERATOR" : operator,                                      
        "MEASUREMENT_SOFTWARE" : software,                             
        "USED_SYSTEM" : "Liverpool System",                               
        "MULTIMETER_MODEL" : "BK2840 for resistance, Arduino Mega for capacitance"                                     
    }  })
    out_dictionary.update({"results" :{
        "CHANNEL_RESISTANCE" : [0, 1, 2],
        "AVERAGE_RESISTANCE" : resistance_average,
        "MINIMUM_RESISTANCE" : resistance_minimum,
        "MAXIMUM_RESISTANCE" : resistance_maximum,
        "EXPECTED_RESISTANCE" : resistance_target,
        "UNCERTAINTY_RESISTANCE" : resistance_stdev,
        "PASS_FAIL_RESISTANCE" : resistance_pass,
        "NUMBER_OF_MEASUREMENTS_RESISTANCE" : resistance_measurements,
        
        "CHANNEL_CAPACITANCE" : [1, 2, 3],
        "AVERAGE_CAPACITANCE" : capacitance_average,
        "MINIMUM_CAPACITANCE" : capacitance_minimum,
        "MAXIMUM_CAPACITANCE" : capacitance_maximum,
        "EXPECTED_CAPACITANCE" : capacitance_target,
        "UNCERTAINTY_CAPACITANCE" : capacitance_stdev,
        "PASS_FAIL_CAPACITANCE" : capacitance_pass,
        "NUMBER_OF_MEASUREMENTS_CAPACITANCE" : capacitance_measurements
        
    }  })
    out_dictionary.update({"attachments" :[
        {
        "title" : os.path.basename(data_file),
        "filename" : data_file,
        "description" : "Raw data file"
        }
    ]  })
    
    #print( json.dumps(out_dictionary, sort_keys=False, indent=4) )
    
    
    
    #########save json file#######################################
    
    df_p = Path(data_file) ##datafile path
    
    file_path = str(df_p.parent) + '\\' + str(component_stage)
    print( file_path )
    
    
    if not os.path.exists( file_path ):
        os.makedirs( file_path )
        print("Created output directory: ", file_path)
    
    
    
    #optional output filename or default from csv
    if out_file_name :
        out_file = str(df_p.parent) +'\\' + component_stage + '\\'+ out_file_name + '.json'
    else :
        out_file = str(df_p.parent) + '\\' + component_stage + '\\'+ db_ID+ '_' + component_stage + '_' + str(run_number) + '.json' 
    print(f"Json output file: {out_file}")
    

    
    with open( out_file, 'w' ) as outfile:
        json.dump(out_dictionary, outfile, indent=4)  #indent makes it human readable
    
    
    
#analyse( "./Flex-File/GPC2149_X_001_A_QA_pre-Reflow_20220125_091849.csv", None, True,\
#        None, 5, None)

    


# In[ ]:


if __name__ == "__main__":
    
    # Parse command line
    parser = argparse.ArgumentParser(description = "Script for analysing pull test csv files")
    parser.add_argument("csv", help = "Full name of csv file containing pull test results or directory of csv files")
    parser.add_argument("--dbID", default = None, help = "Database Component ID")
    parser.add_argument("--alternativeID", default = True, help = "Is alternative ID used? True or False")
    parser.add_argument("--componentStage", default = None, help = "Component stage (database code) ")
    parser.add_argument("--runNumber", default = 0, help = "Run Number for test" )
    parser.add_argument("--outFileName", default = None, help = "Filename for json output.")
    args = parser.parse_args()
    
    
    #!python pull_test_reader Test.csv --dbID 20USB2940 --alternativeID True \
    #        --componentType HYBRID_FLEX --componentStage NEW --runNumber 5 --outFileName Test-New
    
    
    # Opening file
    
    
    
    #analyse (data_file, db_ID, alternative_ID, component_stage, run_number, out_file_name)
    if args.csv.endswith(".csv"):
        print(f"\n>>> Analysing {args.csv} ...")
        analyse(args.csv, args.dbID, args.alternativeID,                args.componentStage, args.runNumber, args.outFileName)          
    elif os.path.isdir(args.csv):
        files = glob(f"{args.csv}/*.csv")
        if not files:
            sys.exit(f"ERROR: No csv files found in directory {args.csv}")
        for f in files:
            print(f"\n>>> Analysing {f} ...")
            analyse(f, args.dbID, args.alternativeID,                 args.componentStage, args.runNumber, args.outFileName)               
    else:
        sys.exit("ERROR: Unknown input format. Please provide a csv file or directory path containing csv files")


# In[ ]:





    


# In[ ]:




