#!/usr/bin/env python
# coding: utf-8

# In[1]:


#based on script from https://gitlab.cern.ch/dengfeng/ITkDBAPI


import os
import json
import tkinter as tk
import tkinter.messagebox as box
from tkinter import filedialog
from tkinter import ttk
import itkdb
from glob import glob
import hybridmodule_etest_uploader

def etest_interface():

    #global variables from function access
    db_password1 = None # Access Code 1
    db_password2 = None # Access Code 2
    global client
    client = None
    global componentID
    componentID = None
    global inPedestalFile, inStrobeDelayFile, inRSFile, inNOFile, in3PGFile
    inPedestalFile =  None
    inStrobeDelayFile = None
    inRSFile = None
    in3PGFile = None
    inNOFile = None

    hybridmodule_test_upload = tk.Toplevel()
    hybridmodule_test_upload.title('Module/hybrid E-Test Uploader Interface')
    hybridmodule_test_upload.geometry("600x620")

    #bring window to top and make active
    hybridmodule_test_upload.lift()
    hybridmodule_test_upload.focus_force()
    hybridmodule_test_upload.grab_set()
    hybridmodule_test_upload.grab_release()

    label_head = tk.Label(hybridmodule_test_upload, text='Database Interface', bg='blue', fg='white', font=('Arial Black', 14, 'bold'))
    label_head.pack(padx = 10, pady = 10)
    
    #####Password verification
    label_pw = tk.Label(hybridmodule_test_upload, text='Please enter database passwords then press the Login button.'
    , wraplength=400 ,width=50, justify='center', bg='deepskyblue') 
    label_pw.pack(padx = 10, pady = 10)

    #  input the passcode 1
    frame_pw1 = tk.Frame(hybridmodule_test_upload)
    frame_pw1.pack()
    label_pw1 = tk.Label(frame_pw1, text='Access Code 1: ')
    label_pw1.pack(side = tk.LEFT, padx = 2, pady = 2)
    entry_pw1 = tk.Entry(frame_pw1)
    entry_pw1['show'] = '*'
    entry_pw1.pack(side = tk.RIGHT, padx = 2, pady = 2)

    #  input the passcode 2
    frame_pw2 = tk.Frame(hybridmodule_test_upload)
    frame_pw2.pack()
    label_pw2 = tk.Label(frame_pw2, text='Access Code 2: ')
    label_pw2.pack(side = tk.LEFT, padx = 2, pady = 2)
    entry_pw2 = tk.Entry(frame_pw2)
    entry_pw2['show'] = '*'
    entry_pw2.pack(side = tk.RIGHT, padx = 2, pady = 2)

    var_login_status = tk.StringVar(hybridmodule_test_upload)
    def pw_read():
        global client
        db_password1 = entry_pw1.get()
        db_password2 = entry_pw2.get()
        if db_password1 and db_password2 :
            try:
                u = itkdb.core.User(access_code1=db_password1, access_code2=db_password2)
                client = itkdb.Client(user = u)
                client.user.authenticate()
                user = client.get('getUser', json={'userIdentity': client.user.identity})
                label_login_status.config(bg="lime")
                label_login_status.config(width=35)
                var_login_status.set("{} {} Login Successfully!".format(user["firstName"], user["lastName"]))
            except Exception:
                label_login_status.config(bg="red")
                label_login_status.config(width=25)
                var_login_status.set("Error! Wrong Passwords!")
                db_password1 = None
                db_password2 = None
                entry_pw1.delete(0,'end')
                entry_pw2.delete(0,'end')
        else: 
            db_password1 = None
            db_password2 = None
            entry_pw1.delete(0,'end')
            entry_pw2.delete(0,'end')
            label_login_status.config(bg="yellow")
            label_login_status.config(width=55)
            var_login_status.set("Warning! None Passwords. Please Enter Passwords!")

    button = tk.Button(hybridmodule_test_upload, text='Login Database', width=15, command = pw_read)
    button.pack(padx = 10, pady = 10)
    frame_login_status = tk.Frame(hybridmodule_test_upload)
    frame_login_status.pack()
    label_login_status = tk.Label(frame_login_status, textvariable=var_login_status)
    label_login_status.pack(side = "top", padx = 10, pady = 10)

    # input the component ID
    frame_component = tk.Frame(hybridmodule_test_upload)
    frame_component.pack()
    label_component = tk.Label(frame_component, text='Component ID: ')
    label_component.pack(side = tk.LEFT, padx = 10, pady = 10)
    component_name = tk.StringVar()
    entry_component = tk.Entry(frame_component, textvariable = component_name)
    entry_component.pack(side = tk.RIGHT, padx = 10, pady = 10)
    
    
    frame_altid= tk.Frame(hybridmodule_test_upload)
    frame_altid.pack()
    altIdLabel = tk.Label(frame_altid, text='Alternative ID?', width = 15 )
    altIdLabel.pack( side=tk.LEFT, padx=10, pady=10)
    altID = tk.BooleanVar()
    radio_alt_no = tk.Radiobutton (frame_altid, text='False', variable = altID, value = False)
    radio_alt_yes = tk.Radiobutton (frame_altid, text='True', variable = altID, value = True)
    radio_alt_yes.select()
    radio_alt_no.pack( side = tk.RIGHT )
    radio_alt_yes.pack( side = tk.RIGHT )
    
    
    

    #####Load all Json Files
    def load_all_jsonfile():
        #if client: 
            #inPath = entry_inpath.get()
            inPath = filedialog.askdirectory(initialdir = "./", title = "Select Directory")
            entry_inpath.insert(0,inPath)
            
            if not os.path.exists(inPath):
                box.showwarning('Warning', inPath, ' does not exist!!!')
                return

            fileList = os.listdir(inPath)
            jsonFiles = {}
            for filename in fileList:
                #filename = inPath+"/"+filename
                filename = os.path.join(inPath, filename)
                if not filename.endswith(".json"): continue
                inFile = open(filename, "r")
                data = json.load(inFile)
                inFile.close()
                #####if data["component"] != component_name.get(): continue ##does not work since ITSDAQ jsons don't have the component name
                #data["component"] = component_name.get() #is not needed at this point
                if data["testType"] not in jsonFiles.keys(): jsonFiles[data["testType"]] = []
                jsonFiles[data["testType"]].append(filename)
            testTypes = jsonFiles.keys()
            
            #print(testTypes)
            
            if len(testTypes) !=4:
                box.showwarning('Warning', 'Not all json files are found!' )
            for testtype in testTypes:
                if "RESPONSE_CURVE" in testtype:
                    if len(jsonFiles[testtype])<2:
                        box.showwarning( 'Warning', '3PG or RS json is not found! Please load manually!')
                    elif len(jsonFiles[testtype])>2:
                        box.showwarning( 'Warning', 'more 3PG or RS json are not found! Please load manually!')
                    else:
                        RSFileList = jsonFiles[testtype]
                        inFile0 = open(RSFileList[0], "r")
                        rsdata0 = json.load(inFile0)
                        inFile0.close()
                        inFile1 = open(RSFileList[1], "r")
                        rsdata1 = json.load(inFile1)
                        inFile1.close()
                        if rsdata0['runNumber']>rsdata1['runNumber']:
                          entry_responsecurve.delete(0,'end')
                          entry_responsecurve.insert(-1, RSFileList[0])
                          entry_3pg.delete(0,'end')
                          entry_3pg.insert(-1, RSFileList[1])
                        else:
                          entry_responsecurve.delete(0,'end')
                          entry_responsecurve.insert(-1, RSFileList[1])
                          entry_3pg.delete(0,'end')
                          entry_3pg.insert(-1, RSFileList[0])
                else:
                    if len(jsonFiles[testtype])==1:
                        if "PEDESTAL_TRIM" in testtype:
                            entry_pedestal.delete(0,'end')
                            entry_pedestal.insert(-1, jsonFiles[testtype][0])
                        elif "STROBE_DELAY" in testtype:
                            entry_strobedelay.delete(0,'end')
                            entry_strobedelay.insert(-1, jsonFiles[testtype][0])
                        elif "NO" in testtype:
                            entry_noiseoccupancy.delete(0,'end')
                            entry_noiseoccupancy.insert(-1, jsonFiles[testtype][0])
                    else:
                        box.showwarning('Warning', 'Please load '+jsonFiles[testtype]+' manually!')

        #else:
        #    box.showwarning( 'Warning', 'Please Login Database!' )

    # input path
    frame_inpath = tk.Frame(hybridmodule_test_upload)
    frame_inpath.pack()
    label_inpath = tk.Label(frame_inpath, text='From Path: ', width=9)
    label_inpath.pack(side = tk.LEFT, padx = 1, pady = 1)
    entry_inpath = tk.Entry(frame_inpath, width=50)
    entry_inpath.pack(side = tk.LEFT, padx = 1, pady = 1)
    load_all_button = tk.Button(frame_inpath, text='Load All', width = 8, command = lambda: load_all_jsonfile())
    load_all_button.pack(side = tk.RIGHT, padx = 1, pady = 1)

    #####Load Json File
    def load_jsonfile(fileName, testtype, entry):
        #if client: 
            json_filename = ""
            if os.path.exists(entry_inpath.get()):
                json_filename =  filedialog.askopenfilename(initialdir = entry_inpath.get(), title = "Select json file",filetypes = (("json file","*.json"),("all files","*.*")))
            else:
                json_filename =  filedialog.askopenfilename(initialdir = os.getcwd(), title = "Select json file",filetypes = (("json file","*.json"),("all files","*.*")))
            if json_filename == '':
                box.showerror( 'Error', 'No file selected.' )
                return
            if not os.path.isfile(json_filename):
                box.showerror( 'Error', 'Json file doesn\'t exists.' )
                return
            fileName = json_filename
            inFile = open(fileName, "r")
            data = json.load(inFile)
            inFile.close()
            entry.delete(0,'end')
            entry.insert(-1, fileName)
        #else:
        #    box.showwarning( 'Warning', 'Please Login Database!.' )

    # pedestal trim
    frame_pedestal = tk.Frame(hybridmodule_test_upload)
    frame_pedestal.pack()  
    entry_pedestal = tk.Entry(frame_pedestal, width = 50)
    entry_pedestal.pack(side = tk.LEFT, padx = 5, pady = 5)
    load_pedestal_button = tk.Button(frame_pedestal, text='PedestalTrim', width = 13, command = lambda: load_jsonfile(inPedestalFile, "PEDESTAL_TRIM", entry_pedestal))
    load_pedestal_button.pack(side = tk.RIGHT, padx = 1, pady = 5)
    # strobe delay
    frame_strobedelay = tk.Frame(hybridmodule_test_upload)
    frame_strobedelay.pack()  
    entry_strobedelay = tk.Entry(frame_strobedelay, width = 50)
    entry_strobedelay.pack(side = tk.LEFT, padx = 1, pady = 5)
    load_strobedelay_button = tk.Button(frame_strobedelay, text='StrobeDelay', width = 13, command = lambda: load_jsonfile(inStrobeDelayFile, "STROBE_DELAY", entry_strobedelay))
    load_strobedelay_button.pack(side = tk.RIGHT, padx = 1, pady = 5)
    # response curve: 3 point gain
    frame_3pg = tk.Frame(hybridmodule_test_upload)
    frame_3pg.pack()  
    entry_3pg = tk.Entry(frame_3pg, width = 50)
    entry_3pg.pack(side = tk.LEFT, padx = 1, pady = 5)
    load_3pg_button = tk.Button(frame_3pg, text='3-Point Gain', width = 13, command = lambda: load_jsonfile(in3PGFile, "RESPONSE_CURVE", entry_3pg))
    load_3pg_button.pack(side = tk.RIGHT, padx = 1, pady = 5)
    # response curve
    frame_responsecurve = tk.Frame(hybridmodule_test_upload)
    frame_responsecurve.pack()  
    entry_responsecurve = tk.Entry(frame_responsecurve, width = 50)
    entry_responsecurve.pack(side = tk.LEFT, padx = 1, pady = 5)
    load_responsecurve_button = tk.Button(frame_responsecurve, text='Response Curve', width = 13, command = lambda: load_jsonfile(inRSFile, "RESPONSE_CURVE", entry_responsecurve))
    load_responsecurve_button.pack(side = tk.RIGHT, padx = 1, pady = 5)
    # noise occupancy
    frame_noiseoccupancy = tk.Frame(hybridmodule_test_upload)
    frame_noiseoccupancy.pack()  
    entry_noiseoccupancy = tk.Entry(frame_noiseoccupancy, width = 50)
    entry_noiseoccupancy.pack(side = tk.LEFT, padx = 1, pady = 5)
    load_noiseoccupancy_button = tk.Button(frame_noiseoccupancy, text='Noise Occupancy', width = 13, command = lambda: load_jsonfile(inNOFile, "NO", entry_noiseoccupancy))
    load_noiseoccupancy_button.pack(side = tk.RIGHT, padx = 1, pady = 5)

    var_upload_status = tk.StringVar(hybridmodule_test_upload)
    def upload():
        global client
        componentID = component_name.get()
        
        if entry_pw1.get() == '' and entry_pw2.get() == '':
            box.showwarning( 'Warning', 'Please Login Database!.' )
            return
        
        if not client :
            box.showwarning( 'Warning', 'Please Login Database!.' )
            return
        
        if componentID == None or componentID == '' : 
            box.showwarning( 'Warning', 'Please Choose a Component name!.' )
            return
        
        inPedestalFile = entry_pedestal.get()
        inStrobeDelayFile = entry_strobedelay.get()
        in3PGFile = entry_3pg.get()
        inRSFile = entry_responsecurve.get()
        inNOFile = entry_noiseoccupancy.get()
        upload_status = True
        failedsteps = ""
        if inPedestalFile != "" and inPedestalFile != None:
            status = hybridmodule_etest_uploader.upload(client, inPedestalFile, componentID, altID.get() )
            if not status: 
                upload_status = False
                failedsteps = failedsteps+ " Pedestal "
        if inStrobeDelayFile != "" and inStrobeDelayFile != None:
            status = hybridmodule_etest_uploader.upload(client, inStrobeDelayFile, componentID, altID.get())
            if not status: 
                upload_status = False
                failedsteps = failedsteps+ " StrobeDelay "
        if in3PGFile != "" and in3PGFile != None:
            status = hybridmodule_etest_uploader.upload(client, in3PGFile, componentID, altID.get())
            if not status: 
                upload_status = False
                failedsteps = failedsteps+ " 3PG "
        if inRSFile != "" and inRSFile != None:
            status = hybridmodule_etest_uploader.upload(client, inRSFile, componentID, altID.get())
            if not status: 
                upload_status = False
                failedsteps = failedsteps+ " RS "
        if inNOFile != "" and inNOFile != None:
            status = hybridmodule_etest_uploader.upload(client, inNOFile, componentID, altID.get())
            if not status: 
                upload_status = False
                failedsteps = failedsteps+ " NO "
        if upload_status:
            var_upload_status.set("Upload Successfully!")
            label_upload.config(bg='green')
            #client = None  #does not make sense
        else:
            var_upload_status.set(failedsteps+"Upload Failed!")
            label_upload.config(bg='red')
            
        component_name.set('')
        entry_inpath.delete(0,'end')
        entry_pedestal.delete(0,'end')
        entry_strobedelay.delete(0,'end')
        entry_3pg.delete(0,'end')
        entry_responsecurve.delete(0,'end')
        entry_noiseoccupancy.delete(0,'end')
            
            
            
    upload_button = tk.Button(hybridmodule_test_upload, text='Upload', width=15, command=upload)
    upload_button.pack(padx = 1, pady = 1)
    frame_upload = tk.Frame(hybridmodule_test_upload)
    frame_upload.pack()  
    label_upload = tk.Label(hybridmodule_test_upload,textvariable = var_upload_status, wraplength=300 ,width=50, justify='center')
    label_upload.pack(padx = 5, pady = 5)

    #Close Window
    def close_window():
        hybridmodule_test_upload.destroy()
        global db_password1, db_password2, client
        client = None
        db_password1 = None   #delete password when window is closed
        db_password2 = None   #delete password whwn window is closed

    exit_button = tk.Button(hybridmodule_test_upload, text='Close', width=15, command=close_window, bg = 'red', )
    exit_button.pack(padx = 5, pady = 5)

    hybridmodule_test_upload.mainloop()

if __name__ == "__main__":
    window = tk.Tk()
    
    quit_button = tk.Button(window, text='Close All', width=25, command=window.destroy, bg = 'red' )
    quit_button.pack(padx = 10, pady = 10)
    
    etest_interface()


# In[ ]:




